import { entireSelectionContainsMark } from '@atlaskit/editor-common/mark';
import { REMOVE_HIGHLIGHT_COLOR } from '@atlaskit/editor-common/ui-color';
import { TextSelection } from '@atlaskit/editor-prosemirror/state';
import { CellSelection } from '@atlaskit/editor-tables/cell-selection';
const getAllUniqueBackgroundColorMarksInRange = (from, to, tr) => {
  const {
    doc
  } = tr;
  const {
    backgroundColor
  } = doc.type.schema.marks;
  const colorMarks = [];
  const colorSet = new Set();
  doc.nodesBetween(from, to, node => {
    if (node.isLeaf) {
      const mark = backgroundColor.isInSet(node.marks);
      if (mark && !colorSet.has(mark.attrs.color)) {
        colorMarks.push(mark);
        colorSet.add(mark.attrs.color);
      }
    }
  });
  return colorMarks;
};
const getAllUniqueBackgroundColorMarksInCellSelection = (selection, tr) => {
  const colorMarks = [];
  const colorSet = new Set();
  selection.forEachCell((cell, cellPos) => {
    const from = cellPos;
    const to = cellPos + cell.nodeSize;
    const marks = getAllUniqueBackgroundColorMarksInRange(from, to, tr);
    marks.forEach(mark => {
      if (!colorSet.has(mark.attrs.color)) {
        colorMarks.push(mark);
        colorSet.add(mark.attrs.color);
      }
    });
  });
  return colorMarks;
};

// For Cell Selections - find first instance of a backgroundColor mark
// if all cells entirely contain this mark, set the color.
// If the selection contains multiple colors, return null
const getColorFromCellSelection = (selection, tr) => {
  const marks = getAllUniqueBackgroundColorMarksInCellSelection(selection, tr);
  if (marks.length > 1) {
    return null;
  }
  const firstColorMark = marks.at(0);
  let foundNonColoredNode = false;
  if (firstColorMark) {
    selection.forEachCell((cell, cellPos) => {
      if (foundNonColoredNode) {
        return;
      }
      const from = cellPos;
      const to = cellPos + cell.nodeSize;
      if (!entireSelectionContainsMark(firstColorMark, tr.doc, from, to)) {
        foundNonColoredNode = true;
      }
    });
  }
  return foundNonColoredNode ? REMOVE_HIGHLIGHT_COLOR : firstColorMark === null || firstColorMark === void 0 ? void 0 : firstColorMark.attrs.color;
};

// All other selections - find the first instance of a backgroundColor mark
// if selection entirely contains this mark, set the color.
// If the selection contains multiple colors, return null
const getColorFromRange = (from, to, tr) => {
  const marks = getAllUniqueBackgroundColorMarksInRange(from, to, tr);
  if (marks.length > 1) {
    return null;
  }
  const firstColorMark = marks.at(0);
  if (firstColorMark && entireSelectionContainsMark(firstColorMark, tr.doc, from, to)) {
    return firstColorMark.attrs.color;
  }
  return REMOVE_HIGHLIGHT_COLOR;
};

// For Cursor selections - set color if it is found in the storedMarks or $cursor.marks
const getColorFromCursor = (selection, tr) => {
  if (!selection.$cursor) {
    return REMOVE_HIGHLIGHT_COLOR;
  }
  const mark = tr.doc.type.schema.marks.backgroundColor.isInSet([...(tr.storedMarks ? tr.storedMarks : []), ...selection.$cursor.marks()]);
  return (mark === null || mark === void 0 ? void 0 : mark.attrs.color) || REMOVE_HIGHLIGHT_COLOR;
};
export const getActiveColor = tr => {
  const {
    selection
  } = tr;
  let color;
  if (selection instanceof CellSelection) {
    color = getColorFromCellSelection(selection, tr);
  } else if (selection instanceof TextSelection && selection.$cursor) {
    color = getColorFromCursor(selection, tr);
  } else {
    color = getColorFromRange(selection.from, selection.to, tr);
  }
  return color === undefined ? REMOVE_HIGHLIGHT_COLOR : color;
};